@@ -2,10 +2,10 @@ package Gimp::Constant;
 
 use Exporter 'import';
 use strict qw(vars);
-our ($VERSION, @EXPORT, @PARAMS, @INXS);
+use vars qw($VERSION);
 
 BEGIN {
-   $VERSION = "2.30_05";
+   $VERSION = 2.3000_01;
 # XS code in Gimp.xs... for now
 #   eval {
 #      require XSLoader;
@@ -17,6 +17,8 @@ BEGIN {
 #   }
 }
 
+use vars qw(@EXPORT @PARAMS @INXS);
+
 my %sub2value;
 
 for my $class (Gimp->enums_get_type_names) {
