@@ -1,18 +1,23 @@
 package Gimp::Lib;
 
 use strict;
-use warnings;
 use vars qw($VERSION @ISA);
 
 BEGIN {
-   $VERSION = "2.30_05";
+   $VERSION = 2.3000_01;
    require XSLoader;
    XSLoader::load Gimp::Lib $VERSION;
    warn "$$-Loading ".__PACKAGE__ if $Gimp::verbose;
 }
 
+use subs qw(
+	gimp_call_procedure		gimp_main	gimp_init
+	gimp_procedural_db_proc_exists	set_trace	gimp_end
+        initialized
+);
+
 sub gimp_init {
-   Gimp::croak Gimp::__("gimp_init not implemented for the Lib interface");
+   Gimp::croak Gimp::_("gimp_init not implemented for the Lib interface");
 }
 
 sub gimp_end {
@@ -75,7 +80,7 @@ sub gimp_drawable_bounds {
    (@b[0,1],$b[2]-$b[0],$b[3]-$b[1]);
 }
 
-warn "$$-Finished loading ".__PACKAGE__ if $Gimp::verbose >= 2;
+warn "$$-Finished loading ".__PACKAGE__ if $Gimp::verbose;
 
 1;
 __END__
@@ -92,7 +97,7 @@ Gimp::Lib - Interface to libgimp (as opposed to Gimp::Net)
 
 This is the package that interfaces to GIMP via the libgimp interface,
 i.e. the normal interface to use with GIMP. You don't normally use this
-module directly, see L<Gimp>.
+module directly, look at the documentation for the package "Gimp".
 
 =head1 AUTHOR
 
