@@ -1,20 +1,73 @@
-package Gimp::Util;
+=head1 NAME
 
+ Gimp::Util - some handy routines for Gimp-Perl users
+
+=head1 SYNOPSIS
+
+ use Gimp;
+ use Gimp::Util;
+
+=head1 DESCRIPTION
+
+Gimp-Perl is nice, but when you have to write everytime 10 lines just to
+get some simple functions done, it very quickly becomes tedious :-/
+
+This module tries to define some functions that aim to automate frequently
+used tasks, i.e. its a sort of catch-all-bag for (possibly) useful macro
+functions.  If you want to add a function just mail the author of the
+Gimp-Perl extension (see below).
+
+In Gimp-Perl (but not in Gimp as seen by the enduser) it is possible to
+have layers that are NOT attached to an image. This is, IMHO a bad idea,
+you end up with them and the user cannot see them or delete them. So we
+always attach our created layers to an image here, too avoid memory leaks
+and debugging times.
+
+These functions try to preserve the current settings like colors, but not
+all do.
+
+These functions can also be handled in exactly the same way as
+PDB-Functions, i.e. the (hypothetical) function C<gimp_image_xyzzy> can be
+called as $image->xyzzy, if the module is available.
+
+The need to explicitly C<use Gimp::Util> will go away in the future.
+
+=head1 FUNCTIONS
+
+=over 4
+
+=cut
+
+package      Gimp::Util;
 use Exporter 'import';
+use vars qw(@EXPORT $VERSION);
 use strict;
-use warnings;
-use File::Basename;
-Gimp->import;
-our @EXPORT_OK = qw(
-  layer_create
-  text_draw
-  image_create_text
-  layer_add_layer_as_mask
-);
+@EXPORT    = qw(
+                layer_create
+                text_draw
+                image_create_text
+                layer_add_layer_as_mask
+               );
+#@EXPORT_OK = qw();
 
-sub __ ($); # declare - defined in Gimp
+import Gimp;
+# manual "import" to shut perl -cw up
+sub __ ($);
 
-our $VERSION = "2.30_05";
+$VERSION = 2.3000_01;
+
+##############################################################################
+=pod
+
+=item C<get_state ()>, C<set_state state>
+
+C<get_state> returns a scalar representing most of gimps global state
+(at the moment foreground colour, background colour, active gradient,
+pattern and brush). The state can later be restored by a call to
+C<set_state>. This is ideal for library functions such as the ones used
+here, at least when it includes more state in the future.
+
+=cut
 
 sub get_state() {
    [
@@ -35,6 +88,15 @@ sub set_state($) {
    Brushes->set_brush($s->[4]);
 }
 
+##############################################################################
+=pod
+
+=item C<layer_create image,name,color,pos>
+
+create a colored layer, insert into image and return layer
+
+=cut
+
 # [12/23/98] v0.0.1 Tels - First version
 sub layer_create {
   my ($image,$name,$color,$pos) = @_;
@@ -48,11 +110,20 @@ sub layer_create {
   $tcol = Gimp->palette_get_background ();
   Gimp->palette_set_background ($color);
   Gimp->drawable_fill ($layer,&BACKGROUND_FILL);
-  Gimp->image_insert_layer($image, $layer, 0, $pos);
+  Gimp->image_add_layer($image, $layer, $pos);
   Gimp->palette_set_background ($tcol); # reset
   $layer;
   }
 
+##############################################################################
+=pod
+
+=item C<text_draw image,layer,text,font,size,fgcolor>
+
+Create a colored text, draw over a background, add to img, ret img.
+
+=cut
+
 # [12/23/98] v0.0.1 Tels - First version
 sub text_draw {
   my ($image,$layer,$text,$font,$size,$fgcolor) = @_;
@@ -74,7 +145,7 @@ sub text_draw {
   Gimp->layer_set_preserve_trans($text_layer, &FALSE);
 
   # add text to image
-  $image->insert_layer($text_layer, 0, 0);
+  Gimp->image_add_layer($image, $text_layer, 0);
   # merge white and text
   Gimp->image_merge_visible_layers ($image,1);
   # cleanup the left over layer (!)
@@ -82,6 +153,16 @@ sub text_draw {
   $layer;
 }
 
+##############################################################################
+=pod
+
+=item C<image_create_text text,font,size,fgcolor,bgcolor>
+
+Create an image, add colored text layer on a background layer, return img.
+
+=cut
+
+# [12/23/98] v0.0.1 Tels - First version
 sub image_create_text {
   my ($text,$font,$size,$fgcolor,$bgcolor) = @_;
   my $tcol; # temp. color
@@ -119,6 +200,16 @@ sub image_create_text {
   $image;
   }
 
+##############################################################################
+=pod
+
+=item C<layer_add_layer_as_mask image,layer,layermask>
+
+Take a layer and add it as a mask to another layer, return mask.
+
+=cut
+
+# [12/23/98] v0.0.1 Tels - First version
 sub layer_add_layer_as_mask {
   my ($image,$layer,$layer_mask) = @_;
   my $mask;
@@ -133,6 +224,33 @@ sub layer_add_layer_as_mask {
   $mask;
   }
 
+##############################################################################
+# all functions below are by Marc Lehmann
+=pod
+
+=item C<gimp_image_layertype $alpha>
+
+returns the corresponding layer type for an image, alpha controls wether the layer type
+is with alpha or not. Example: imagetype: RGB -> RGB_IMAGE (or RGBA_IMAGE).
+
+=item C<gimp_layer2imagetype $layertype>
+
+returns the corresponding layer type for an image, alpha controls wether the layer type
+is with alpha or not. Example: imagetype: RGB -> RGB_IMAGE (or RGBA_IMAGE).
+
+=item C<gimp_image_add_new_layer $image,$index,$fill_type,$alpha>
+
+creates a new layer and adds it at position $index (default 0) to the
+image, after filling it with gimp_drawable_fill $fill_type (default
+BG_IMAGE_FILL). If $alpha is non-zero (default 1), the new layer has
+alpha.
+
+=item C<gimp_image_set_visible $image,@layers>, C<gimp_image_set_invisible $image,@layers>
+
+mark the given layers visible (invisible) and all others invisible (visible).
+
+=cut
+
 sub gimp_image_layertype {
    my($type,$alpha) = ($_[0]->base_type,$_[1]);
    $type == &RGB     ? $alpha ? &RGBA_IMAGE     : &RGB_IMAGE     :
@@ -154,14 +272,9 @@ sub gimp_layer2imagetype {
 
 sub gimp_image_add_new_layer {
    my ($image,$index,$filltype,$alpha)=@_;
-   my $layer = Gimp::Layer->new(
-      $image, $image->width, $image->height,
-      $image->layertype (defined $alpha ? $alpha : 1),
-      join(':', map { basename($_) } (caller 2)[1,2]),
-      100, &NORMAL_MODE
-   );
+   my $layer = new Gimp::Layer ($image, $image->width, $image->height, $image->layertype (defined $alpha ? $alpha : 1), join(":",(caller)[1,2]), 100, &NORMAL_MODE);
    $layer->fill (defined $filltype ? $filltype : &BACKGROUND_FILL);
-   $image->insert_layer ($layer, 0, $index*1);
+   $image->add_layer ($layer, $index*1);
    $layer;
 }
 
@@ -169,7 +282,7 @@ sub gimp_image_set_visible {
    my $image = shift;
    my %layers; @layers{map $$_,@_}=(1) x @_;
    for ($image->get_layers) {
-      $_->set_visible ($layers{$$_} // 0);
+      $_->drawable_set_visible ($layers{$$_});
    }
 }
 
@@ -177,100 +290,17 @@ sub gimp_image_set_invisible {
    my $image = shift;
    my %layers; @layers{map $$_,@_}=(1) x @_;
    for ($image->get_layers) {
-      $_->set_visible (!$layers{$$_} // 0);
+      $_->drawable_set_visible (!$layers{$$_});
    }
 }
 
-1;
-__END__
-=head1 NAME
-
-Gimp::Util - Handy routines for Gimp-Perl users
-
-=head1 SYNOPSIS
-
- use Gimp;
- use Gimp::Util;
-
-=head1 DESCRIPTION
-
-Gimp-Perl is nice, but when you have to write everytime 10 lines just to
-get some simple functions done, it very quickly becomes tedious :-/
-
-This module tries to define some functions that aim to automate frequently
-used tasks, i.e. its a sort of catch-all-bag for (possibly) useful macro
-functions.  If you want to add a function just mail the author of the
-Gimp-Perl extension (see below).
-
-In Gimp-Perl (but not in Gimp as seen by the enduser) it is possible to
-have layers that are NOT attached to an image. This is, IMHO a bad idea,
-you end up with them and the user cannot see them or delete them. So we
-always attach our created layers to an image here, too avoid memory leaks
-and debugging times.
-
-These functions try to preserve the current settings like colors, but not
-all do.
-
-These functions can also be handled in exactly the same way as
-PDB-Functions, i.e. the (hypothetical) function C<gimp_image_xyzzy> can be
-called as $image->xyzzy, if the module is available.
-
-The need to explicitly C<use Gimp::Util> will go away in the future.
-
-=head1 FUNCTIONS
-
-=over 4
-
-=item C<get_state ()>, C<set_state state>
-
-C<get_state> returns a scalar representing most of gimps global state
-(at the moment foreground colour, background colour, active gradient,
-pattern and brush). The state can later be restored by a call to
-C<set_state>. This is ideal for library functions such as the ones used
-here, at least when it includes more state in the future.
-
-=item C<layer_create image,name,color,pos>
-
-create a colored layer, insert into image and return layer
-
-=item C<text_draw image,layer,text,font,size,fgcolor>
-
-Create a colored text, draw over a background, add to img, ret img.
-
-=item C<image_create_text text,font,size,fgcolor,bgcolor>
-
-Create an image, add colored text layer on a background layer, return img.
-
-=item C<layer_add_layer_as_mask image,layer,layermask>
-
-Take a layer and add it as a mask to another layer, return mask.
-
-=item C<gimp_image_layertype $alpha>
-
-returns the corresponding layer type for an image, alpha controls wether the layer type
-is with alpha or not. Example: imagetype: RGB -> RGB_IMAGE (or RGBA_IMAGE).
-
-=item C<gimp_layer2imagetype $layertype>
-
-returns the corresponding layer type for an image, alpha controls wether the layer type
-is with alpha or not. Example: imagetype: RGB -> RGB_IMAGE (or RGBA_IMAGE).
-
-=item C<$image-E<gt>add_new_layer($index,$fill_type,$alpha)>
-
-creates a new layer and adds it at position $index (default 0) to the
-image, after filling it with gimp_drawable_fill $fill_type (default
-BG_IMAGE_FILL). If $alpha is non-zero (default 1), the new layer has
-alpha. Makes up a name based on the 2nd-level caller filename and line.
-
-=item C<$image-E<gt>set_visible(@layers)>, C<$image-E<gt>set_invisible(@layers)>
-
-mark the given layers visible (invisible) and all others invisible (visible).
+=pod
 
 =back
 
 =head1 AUTHOR
 
-Various, version 1.000 written mainly by Tels (http://bloodgate.com/). The
-author of the Gimp-Perl extension (contact him to include new functions)
-is Marc Lehmann <pcg@goof.com>
+Various, version 1.000 written mainly by Tels (http://bloodgate.com/). The author
+of the Gimp-Perl extension (contact him to include new functions) is Marc
+Lehmann <pcg@goof.com>
 
